import { world, system, MolangVariableMap, } from "@minecraft/server";
import * as structures from "./structures";
import { boundryBoxList, globals } from "./globals";
import { tutorial } from "./tutorial";
import { suscribeEvents } from "./events";
let ready = false;
let player;
let currentSpawnPoint;
function updateCamera(player) {
    const { x, y, z } = player.location;
    const scoreboard = world.scoreboard;
    const upper = scoreboard.getObjective("upperDistance")?.getScore("-253403070434");
    const lateral = scoreboard.getObjective("lateralDistance")?.getScore("-253403070434");
    player.camera.setCamera("minecraft:free", {
        location: { x: x, y: y + upper + 0.5, z: z - lateral },
        easeOptions: { easeTime: 0.15, easeType: "InOutSine" },
    });
}
function updateSpawnPoint(player) {
    const newSpawnPoint = player.getSpawnPoint();
    if (newSpawnPoint) {
        if (newSpawnPoint.x != currentSpawnPoint.x || newSpawnPoint.y != currentSpawnPoint.y) {
            player.runCommandAsync(`setblock ${currentSpawnPoint.x} -64 137 border_block`);
            player.runCommandAsync(`fill ${currentSpawnPoint.x} -64 137 ${currentSpawnPoint.x} 319 137 barrier`);
            currentSpawnPoint = newSpawnPoint;
            player.runCommandAsync(`setblock ${currentSpawnPoint.x} -64 136 border_block`);
            player.runCommandAsync(`setblock ${currentSpawnPoint.x} ${--currentSpawnPoint.y} 137 bedrock`);
            player.runCommandAsync(`setblock ${currentSpawnPoint.x} ${++currentSpawnPoint.y} 137 heavy_weighted_pressure_plate`);
            player.runCommandAsync(`setblock ${currentSpawnPoint.x} ${++currentSpawnPoint.y} 137 air`);
            player.runCommandAsync(`setblock ${currentSpawnPoint.x} ${currentSpawnPoint.y + 1} 137 air`);
            player.runCommandAsync(`setblock ${currentSpawnPoint.x} -64 137 air`);
            --currentSpawnPoint.y;
            if (newSpawnPoint.z < 138) {
                newSpawnPoint.z = 138;
                player.setSpawnPoint(newSpawnPoint);
            }
        }
    }
}

function main() {
    if (!ready) {
        world.getAllPlayers().map((p) => {
            player = p;
            const scoreboard = world.scoreboard;
            const molang = new MolangVariableMap();
            molang.setFloat("variable.lifetime", 20);
            globals.molangParticles = molang;
            if (!scoreboard.getObjective("firstJoin")) {
                scoreboard.addObjective("firstJoin", "dummy");
                player.teleport({ x: 10853, y: -47, z: 160 }, { rotation: { x: 0, y: 90 } });

                scoreboard.getObjective("upperDistance")?.setScore("-253403070434", 2);
                scoreboard.getObjective("lateralDistance")?.setScore("-253403070434", 10);
                scoreboard.getObjective("cam2D")?.setScore("-253403070434", 1);
                globals.camera2D = scoreboard.getObjective("cam2D")?.getScore("-253403070434"),
                globals.holderL = world.getDimension("overworld").spawnEntity("dv:holder", { x: 10841, y: -60, z: 160 });
                globals.holderR = world.getDimension("overworld").spawnEntity("dv:holder2", { x: 10841, y: -60, z: 160 });
                globals.holderL.teleport({ x: 11019, y: -33, z: 138 });
                globals.holderR.teleport({ x: 10716, y: -33, z: 138 });
                player.runCommandAsync("gamerule domobspawning false");

                globals.boundryBox = world.getDimension("overworld").spawnEntity("dv:box", { x: 10841, y: -50, z: 160 });
                player.setSpawnPoint({ dimension: player.dimension, x: 10850, y: 128, z: 138.5 });
                currentSpawnPoint = player.getSpawnPoint();
                tutorial(player).then(() => {
                    system.clearRun(mainRun);
                    structures.loadBiomes();
                    structures.loadNether();
                    suscribeEvents(player);
                    gameLoop();
                });
            }
            else {
                system.clearRun(mainRun);
                structures.loadBiomes();
                structures.loadNether();
                suscribeEvents(player);
                currentSpawnPoint = player.getSpawnPoint();
                const dim = player.dimension.id;
                if (dim == "minecraft:overworld") {
                    world.getDimension('overworld').getEntities().some((entity) => {
                        if (entity.typeId === "dv:holder")
                            globals.holderL = entity;
                        else if (entity.typeId === "dv:holder2")
                            globals.holderR = entity;
                        else if (entity.typeId === "dv:box")
                            globals.boundryBox = entity;

                        return globals.holderL && globals.holderR && globals.boundryBox;
                    });
                } else if (dim == "minecraft:nether") {
                    globals.boundryBox = world.getDimension('nether').getEntities().find((entity) => {
                        return entity.typeId === "dv:box";
                    });
                }
                gameLoop();
            }
            ready = true;
        });
    }
    if (ready && globals.camera2D) {
        const velocity = player.getVelocity();
        if (velocity.x === 0 && velocity.y === 0 && velocity.z === 0) {
            if (system.currentTick % 3 === 0)
                updateCamera(player);
        }
        else
            updateCamera(player);
    }
    system.run(main);
}
function gameLoop() {
    if (ready && globals.camera2D) {
        console;
        const velocity = player.getVelocity();
        if (velocity.x === 0 && velocity.y === 0 && velocity.z === 0)
            if (system.currentTick % 3 === 0)
                updateCamera(player);
            else
                updateCamera(player);
        if (globals.boundryBoxActive == 1) {
            const viewEntity = player.getEntitiesFromViewDirection()[0]?.entity;
            const viewBlock = player.getBlockFromViewDirection()?.block.location;
            if (viewEntity && viewEntity != globals.boundryBox) {
                const entityLocation = viewEntity.location;
                globals.boundryBox?.teleport({
                    x: entityLocation.x - 0.5,
                    y: entityLocation.y + boundryBoxList(viewEntity.typeId),
                    z: entityLocation.z - 0.55,
                });
            }
            else if (viewBlock)
                globals.boundryBox?.tryTeleport(viewBlock);
        }
        else
            globals.boundryBox?.teleport({ x: 10841, y: -48, z: 160 });
    }
    if (system.currentTick % 30 == 0) {
        const dim = player.dimension.id;
        if (dim == "minecraft:overworld") {
            updateSpawnPoint(player);
            structures.generateBiome(player);
            try {
                world
                    .getDimension("overworld")
                    .spawnParticle("dv:halo", { x: currentSpawnPoint.x + 0.5, y: currentSpawnPoint.y, z: 137.4 }, globals.molangParticles);
            }
            catch {
                // Avoid error message
                // Particles are out of render chunk and throws and error
            }
        }
        else if (dim == "minecraft:nether")
            structures.generateNetherBiome(player);    
    }
    system.run(gameLoop);
}
let mainRun = system.run(main);