import { world } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { globals } from "./globals";
export function menu(player) {
    const form = new ActionFormData();
    form.title("Menu");
    form.body("Choose an option");
    form.button("Visuals", "textures/ui/visuals_icon");
    form.button("Achievements", "textures/ui/achievements_icon");
    form.show(player).then((result) => {
        if (result.selection == 0) {
            visualsMenu(player);
        }
        if (result.selection == 1) {
            achievementMenu(player);
        }
    });
}
function visualsMenu(player) {
    const formVis = new ActionFormData();
    formVis.title("Visual options");
    formVis.button("First Person View");
    formVis.button("2D View");
    formVis.button("Enable Coordinates");
    formVis.button("Disable Coordinates");
    formVis.show(player).then((res) => {
        if (res.selection == 0) {
            if (globals.camera2D) {
                player.camera.setCamera("minecraft:first_person");
                world.scoreboard.getObjective("cam2D")?.removeParticipant("-253403070434");
                globals.camera2D = undefined;
                globals.camDistanceX = player.location.x;
                globals.camDistanceY = player.location.y;
                globals.boundryBoxActive = 0;
                globals.boundryBox?.teleport({ x: 10841, y: 125, z: 242 });
                player.sendMessage("Camera view has changed to first person");
            }
            else {
                player.sendMessage("You are already on first person view");
            }
        }
        else if (res.selection == 1) {
            secondPersonViewMenu(player);
        }
        else if (res.selection == 2) {
            player.runCommandAsync("gamerule showcoordinates true");
        }
        else if (res.selection == 3) {
            player.runCommandAsync("gamerule showcoordinates false");
        }
    });
}
export function secondPersonViewMenu(player) {
    const optViewForm = new ModalFormData();
    const scoreboard = world.scoreboard;
    optViewForm.title("2D View Options");
    optViewForm.slider("Lateral View", 5, 15, 1, scoreboard.getObjective("lateralDistance")?.getScore("-253403070434"));
    optViewForm.slider("Upper View", 1, 3, 1, scoreboard.getObjective("upperDistance")?.getScore("-253403070434"));
    if (globals.boundryBoxActive == 0)
        optViewForm.toggle("Visual Range Box", true);
    else
        optViewForm.toggle("Visual Range Box", true);
    optViewForm.show(player).then((response) => {
        if (response.formValues) {
            const lateral = response.formValues[0];
            const upper = Math.round(response.formValues[1]);
            if (response.formValues[2]) {
                globals.boundryBoxActive = 1;
                scoreboard.getObjective("boundryBoxActive")?.setScore("-253403070434", 1);
            }
            else {
                globals.boundryBoxActive = 0;
                scoreboard.getObjective("boundryBoxActive")?.setScore("-253403070434", 0);
            }
            scoreboard.getObjective("lateralDistance")?.setScore("-253403070434", lateral);
            scoreboard.getObjective("upperDistance")?.setScore("-253403070434", upper);
            scoreboard.getObjective("cam2D")?.setScore("-253403070434", 1);
            globals.camera2D = 1;
            const { x, y, z } = player.location;
            player.camera.setCamera("minecraft:free", {
                location: { x: x, y: y, z: z - lateral },
                easeOptions: { easeTime: 0.15, easeType: "InOutSine" },
            });
            player.sendMessage("Camera view has been updated");
        }
    });
}
function achievementMenu(player) {
    const formGam = new ActionFormData();
    const scoreboard = world.scoreboard;
    formGam.title("Achievements");
    formGam.body("Click on the achievement holding the item to win a reward!");
    const achievements = [
        { objective: "a1", text: "Get a Stone pickaxe", texture: "textures/items/stone_pickaxe", item: "minecraft:stone_pickaxe" },
        { objective: "a2", text: "Get a Fishing Rod", texture: "textures/items/fishing_rod_uncast", item: "minecraft:fishing_rod" },
        { objective: "a3", text: "Get a Bow", texture: "textures/items/bow_standby", item: "minecraft:bow" },
        { objective: "a4", text: "Get a Compass", texture: "textures/items/compass_item", item: "minecraft:compass" },
        { objective: "a5", text: "Get a Book", texture: "textures/items/book_normal", item: "minecraft:book" },
        { objective: "a6", text: "Get a Diamond Axe", texture: "textures/items/diamond_axe", item: "minecraft:diamond_axe" },
        { objective: "a7", text: "Get a Trident", texture: "textures/items/trident", item: "minecraft:trident" },
        { objective: "a8", text: "Get a Stone Bricks", texture: "textures/blocks/clay", item: "minecraft:clay" },
        { objective: "a9", text: "Get a Scaffolding", texture: "textures/blocks/scaffolding_top", item: "minecraft:scaffolding" },
        { objective: "a10", text: "Get a Cherry Planks", texture: "textures/blocks/cherry_planks", item: "minecraft:cherry_planks" },
        { objective: "a11", text: "Get a Jack'o Lantern", texture: "textures/blocks/pumpkin_face_on", item: "minecraft:jack_o_lantern" },
        { objective: "a12", text: "Get a Red Mushroom Block", texture: "textures/blocks/mycelium_side", item: "minecraft:red_mushroom_block" },
        { objective: "a13", text: "Get a Prismarine Brick", texture: "textures/blocks/sea_lantern2", item: "minecraft:prismarine_bricks" },
        { objective: "a14", text: "Get a Nether Wart Block", texture: "textures/blocks/nether_wart_block", item: "minecraft:nether_wart_block" },
        { objective: "a15", text: "Get a Milk Bucket", texture: "textures/items/bucket_milk", item: "minecraft:milk_bucket" },
        { objective: "a16", text: "Get a Biscuit", texture: "textures/items/cookie", item: "minecraft:cookie" },
        { objective: "a17", text: "Get a Rabbit Foot", texture: "textures/items/rabbit_foot", item: "minecraft:rabbit_foot" },
        { objective: "a18", text: "Get a Amethyst Shard", texture: "textures/items/amethyst_shard", item: "minecraft:amethyst_shard" },
        { objective: "a19", text: "Get a Slime Ball", texture: "textures/items/slimeball", item: "minecraft:slime_ball" },
        { objective: "a20", text: "Get a Honey Bottle", texture: "textures/items/honey_bottle", item: "minecraft:honey_bottle" },
        { objective: "a21", text: "Get a Blaze Rod", texture: "textures/items/blaze_rod", item: "minecraft:blaze_rod" },
    ];
    const availableAchievements = achievements.filter(ach => scoreboard.getObjective(ach.objective)?.getScore("-253403070434") === 0);
    availableAchievements.forEach(achievement => {
        formGam.button(achievement.text, achievement.texture);
    });
    if (availableAchievements.length > 0) {
        formGam.show(player).then((res) => {
            if (!res.canceled && res.selection !== undefined) {
                const selectedAchievement = availableAchievements[res.selection];
                checkInventoryAndGiveReward(player, selectedAchievement.item, selectedAchievement.objective);
            }
        });
    }
    else {
        player.sendMessage("You completed all achievements");
    }
}
function checkInventoryAndGiveReward(player, itemType, achievementId) {
    const inventory = player.getComponent("inventory");
    const inventorySize = 36;
    for (let i = 0; i < inventorySize; i++) {
        if (inventory.container?.getItem(i)?.typeId == itemType) {
            for (let blocks = 0; blocks < 5; blocks++) {
                const { x, y } = player.location;
                const temp_x = getRandomArbitrary(x + 100, x - 100);
                let temp_y = getRandomArbitrary(y + 30, y - 30);
                let block = world.getDimension("overworld").getBlock({ x: temp_x, y: temp_y, z: 138 });
                while (!block?.permutation.matches("air") && !block?.permutation.matches("dirt")) {
                    block = world.getDimension("overworld").getBlock({ x: temp_x, y: ++temp_y, z: 138 });
                }
                player.runCommandAsync(`setblock ${temp_x} ${temp_y} 138 dv:lucky_block`);
                i++;
            }
            player.sendMessage("5 lucky blocks have been summoned, go find them!");
            world.scoreboard.getObjective(achievementId)?.removeParticipant("-253403070434");
            break;
        }
    }
}
function getRandomArbitrary(min, max) {
    return Math.random() * (max - min) + min;
}